﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using Infragistics.Olap.FlatData;
using Microsoft.Win32;
using System.IO;
using Infragistics.Documents.Excel;
using Infragistics.Controls.Grids;
using Infragistics.Olap.Data;
using Infragistics.Olap;
using Infragistics.Controls.Grids.Primitives;

namespace FromDataSet
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataHelper dh = new DataHelper();
            DataSet ds = dh.GetDataFromDB();
            FlatDataSource flds = dh.CreateFlatDataSource(ds.Tables[0]);
            myPivotGrid.DataSource = flds;
            myPivotSelector.DataSource = flds;
            myPivotGrid.Loaded += new RoutedEventHandler(myPivotGrid_Loaded);
           
        }

        void myPivotGrid_Loaded(object sender, RoutedEventArgs e)
        {
            myPivotGrid.MeasuresDropControl.DataContextChanged += new DependencyPropertyChangedEventHandler(MeasuresDropControl_DataContextChanged);
        }

        void MeasuresDropControl_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            
        }

      



        void rc_Drop(object sender, DragEventArgs e)
        {
           
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            //if data is still not loaded give a warning.
            if (this.myPivotGrid.GridLayout == null ||
                this.myPivotGrid.GridLayout.Rows == null ||
                this.myPivotGrid.GridLayout.Columns == null)
            {
                MessageBox.Show("Loaded");
                return;
            }

            this.SaveExport(this.CreateWorkbook());
        }

        

        private Workbook CreateWorkbook()
        {
            
            int TopHeaderHeight = this.myPivotGrid.GridLayout.PrivateRowsForColumnPanel.Count;

            int LeftHeaderWidth = this.myPivotGrid.GridLayout.PrivateColumnsForRowPanel.Count;
            //Create the blank workbook and the sheet in it that will contain the PivotGrid data
            Workbook dataWorkbook = new Workbook();
            Worksheet sheetOne = dataWorkbook.Worksheets.Add("Data from PivotGrid");

            //Freeze header rows and columns
            sheetOne.DisplayOptions.PanesAreFrozen = true;
            sheetOne.DisplayOptions.FrozenPaneSettings.FrozenRows = TopHeaderHeight;
            sheetOne.DisplayOptions.FrozenPaneSettings.FrozenColumns = LeftHeaderWidth;

            //Set default width for the columns
            sheetOne.DefaultColumnWidth = 5000;


            int ColumnSpan, ColumnId, RowSpan, RowId;
            string CellValue;

          
            // Build Column Header
            foreach (PivotColumnHeaderCell cell in this.myPivotGrid.GridLayout.ColumnHeaderCells)
            {
                ColumnSpan = cell.RealColumnSpan;
                ColumnId = this.myPivotGrid.GridLayout.Columns
                    .IndexOf(cell.Column as PivotDataColumn)
                    + LeftHeaderWidth;

                RowSpan = cell.RowSpan;
                RowId = this.myPivotGrid.GridLayout
                    .PrivateRowsForColumnPanel.IndexOf(cell.Row as PivotHeaderRow);

                CellValue = cell.Member.Caption;
                if (cell.Member.IsTotal && cell.IsToggleVisible == false)
                    CellValue += " " + "Total";

                if (ColumnSpan > 1 || RowSpan > 1)
                    sheetOne.MergedCellsRegions.Add(
                        RowId,
                        ColumnId,
                        RowId + RowSpan - 1,
                        ColumnId + ColumnSpan - 1
                        );

                this.SetCellValue(sheetOne.Rows[RowId].Cells[ColumnId], CellValue, "TopHeader");
            }
           

            
            // Build Row Header
            foreach (PivotRowHeaderCell cell in this.myPivotGrid.GridLayout.RowHeaderCells)
            {
                ColumnSpan = cell.ColumnSpan;
                ColumnId = this.myPivotGrid.GridLayout
                    .PrivateColumnsForRowPanel.IndexOf(cell.Column as PivotHeaderColumn);

                RowSpan = cell.RealRowSpan;
                RowId = this.myPivotGrid.GridLayout
                    .Rows.IndexOf(cell.Row as PivotDataRow) + TopHeaderHeight;

                CellValue = cell.Member.Caption;
                if (cell.Member.IsTotal && cell.IsToggleVisible == false)
                    CellValue += " " + "Total";

                if (ColumnSpan > 1 || RowSpan > 1)
                    sheetOne.MergedCellsRegions.Add(
                        RowId,
                        ColumnId,
                        RowId + RowSpan - 1,
                        ColumnId + ColumnSpan - 1
                        );

                this.SetCellValue(sheetOne.Rows[RowId].Cells[ColumnId], CellValue, "LeftHeader");
            }
           

            // Build Data
            int currentRow = 0;
            foreach (PivotDataRow row in this.myPivotGrid.GridLayout.Rows)
            {
                int currentCell = 0;
                foreach (PivotCell cell in row.Cells)
                {
                    string currentCellValue;
                    if (cell.Data != null)
                    {
                        currentCellValue = (cell.Data as ICell).FormattedValue;
                    }
                    else
                    {
                        currentCellValue = string.Empty;
                    }
                    this.SetCellValue(
                        sheetOne.Rows[currentRow + TopHeaderHeight]
                            .Cells[currentCell + LeftHeaderWidth],
                        currentCellValue,
                        "Data");
                    currentCell++;
                }
                currentRow++;
            }

            //return the filled in workbook
            return dataWorkbook;
        }

        //Set the value and format each cell
        private void SetCellValue(WorksheetCell cell, string CellValue, string CellType)
        {
            cell.Value = CellValue;
            cell.CellFormat.ShrinkToFit = ExcelDefaultableBoolean.True;
            cell.CellFormat.VerticalAlignment = VerticalCellAlignment.Center;
            cell.CellFormat.Alignment = HorizontalCellAlignment.Center;

            //if (CellType == "TopHeader")
            //{
            //    cell.CellFormat.FillPatternForegroundColor = new Color()
            //    {
            //        A = 255,
            //        R = 181,
            //        G = 212,
            //        B = 240
            //    };
            //    cell.CellFormat.FillPattern = FillPatternStyle.Solid;
            //}
            //else if (CellType == "LeftHeader")
            //{
            //    cell.CellFormat.FillPatternForegroundColor = new Color()
            //    {
            //        A = 255,
            //        R = 200,
            //        G = 212,
            //        B = 240
            //    };
            //    cell.CellFormat.FillPattern = FillPatternStyle.Solid;
            //}
            //else if (CellType == "Data")
            //{
            //    cell.CellFormat.FillPatternForegroundColor = new Color()
            //    {
            //        A = 255,
            //        R = 242,
            //        G = 244,
            //        B = 249
            //    };
            //    cell.CellFormat.FillPattern = FillPatternStyle.Solid;
            //}
            //else
            //{
            //    MessageBox.Show("Cell Type Not Located");
            //}
        }

        //Open a dialog and prompt the user to save the Excel workbook with the exported data
        private void SaveExport(Workbook dataWorkbook)
        {
            SaveFileDialog dialog = new SaveFileDialog
            {
                Filter = "Excel files|*.xls",
                DefaultExt = "xls"
            };

            bool? showDialog = dialog.ShowDialog();
            if (showDialog == true)
            {
                using (Stream exportStream = dialog.OpenFile())
                {
                    dataWorkbook.Save(exportStream);
                    exportStream.Close();
                }
            }
        }

        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
           
            MenuItem mi = (MenuItem)sender;
            
           
            FlatDataSource fds = (FlatDataSource)myPivotGrid.DataSource;
            FlatMeasure fm = null;
            if (fds.Measures.Count > 1)
            {
                PivotCellControl pcc = mi.DataContext as PivotCellControl;
                PivotCell pc = (PivotCell)pcc.Cell;
                BaseMeasureViewModel bmvm = null;
                string[] headers = pc.DataColumn.HeaderText.Split('/');
                for (int i = 0; i < headers.Count(); i++)
                {
                    string mHeader = headers[i];
                    bmvm = (BaseMeasureViewModel)(from m in fds.Measures
                                                                       where m.Caption == mHeader
                                                                       select m).FirstOrDefault();
                    if (bmvm != null && bmvm.Caption == mHeader)
                        break;
                }
                fm = (FlatMeasure) bmvm.Measure;


            }
            else
            {
                fm = (FlatMeasure)((BaseMeasureViewModel)fds.Measures[0]).Measure;
            }

            if (fm != null)
            {
                switch (mi.Header.ToString())
                {
                    case "Min":
                        fm.Aggregator = new DecimalMinAggregator();
                        break;
                    case "Max":
                        fm.Aggregator = new DecimalMaxAggregator();
                        break;
                    case "Avg":
                        fm.Aggregator = new DecimalAverageAggregator();
                        break;
                }


                myPivotGrid.DataSource.RefreshGrid();
            }

        }

        private void ContextMenu_ContextMenuOpening(object sender, ContextMenuEventArgs e)
        {

        }

        private void MenuSum_Click(object sender, RoutedEventArgs e)
        {

        }

        private void MenuMax_Click(object sender, RoutedEventArgs e)
        {

        }


       
    }
}
